@@ -15,10 +15,84 @@
 
 __all__ = [
     "implements", "accepts", "rejects",
-    "requires", "returns", "exposes"
+    "requires", "returns", "exposes",
+    "disallows", "throws"
     ]
 from functools import wraps
 
+ARG_NOT_EXPOSED  = "Named argument {0} is not exposed."
+ARG_IS_REQUIRED  = "The argument {0} is required."
+MUST_NOT_BE_TYPE = "Argument {0} must not be of type {1}."
+MUST_RETURN_TYPE = "Function must return type[s]: {0}."
+MUST_ACCEPT_TYPE = "Argument {1} must be of type {1}."
+DOESNT_IMPLEMENT = "Object doesn't implement method {0}."
+EXCEPTION_TYPE   = "Raised exception must be of type {0}."
+
+def throws(*exceptions):
+    """
+    Declares that the function can only
+    raise the supplied exceptions- any
+    other raised will cause in a RuntimeError.
+    This is only useful but isn't used
+    solely for testing. Usage example::
+
+        >>> @throws(ValueError)
+        ... def f(div):
+        ...     if div == 0:
+        ...         raise ValueError
+        ...     return 5/div
+
+    You may not want to actually use this
+    signature in production since it is
+    actually quite slow (exception handling
+    does come at a price!).
+    """
+    def wrapper(f):
+        @wraps(f)
+        def inner(*args, **kwargs):
+            try:
+                return f(*args, **kwargs)
+            except Exception as err:
+                # apparently we cannot just do ``err in
+                # exceptions`` since the err variables is
+                # an instance
+                if err.__class__ not in exceptions:
+                    raise RuntimeError(EXCEPTION_TYPE.format(exceptions))
+                raise
+        return inner
+    return wrapper
+
+def disallows(*not_allowed):
+    """
+    Declares that the wrapped function doesn't
+    allow any keyword arguments of the given
+    name- the logical negation of ``exposes``.
+    Usage example::
+
+        >>> @disallows('bits')
+        ... def f(bits=0):
+        ...     return file.read(bits)
+
+    Similar to the exposes function, if the
+    "*" is the only argument passed in, the
+    decorator will return a function that
+    will only take in positional arguments.
+    """
+    def wrapper(f):
+        if not_allowed == ('*',):
+            @wraps(f)
+            def inner(*args):
+                return f(*args)
+            return inner
+        @wraps(f)
+        def inner(*args, **kwargs):
+            for key, value in kwargs.items():
+                if key in not_allowed:
+                    raise TypeError(ARG_NOT_EXPOSED.format(key))
+            return f(*args, **kwargs)
+        return inner
+    return wrapper
+
 def exposes(*allowed):
     """
     Declares that the function can only be invoked
@@ -34,18 +108,22 @@ def exposes(*allowed):
         File <stdin> line ?:
             f(u=3)
         TypeError
+
+    If "*" is the only passed in argument, this
+    decorator will return a function that can only
+    be called with keyword arguments.
     """
     def wrapper(f):
-        if allowed == tuple('*'):
+        if allowed == ('*',):
             @wraps(f)
-            def function(**kwargs):
+            def inner(**kwargs):
                 return f(**kwargs)
-            return function
+            return inner
         @wraps(f)
         def inner(*args, **kwargs):
-            for key, value in kwargs.items():
+            for key, _ in kwargs.items():
                 if key not in allowed:
-                    raise TypeError
+                    raise TypeError(ARG_NOT_EXPOSED.format(key))
             return f(*args, **kwargs)
         return inner
     return wrapper
@@ -68,7 +146,7 @@ def implements(*items):
             for object_ in compl:
                 for method in items:
                     if not hasattr(object_, method):
-                        raise TypeError
+                        raise TypeError(DOESNT_IMPLEMENT.format(method))
             return f(*args, **kwargs)
         return inner
     return wrapper
@@ -97,7 +175,7 @@ def requires(*items):
         def inner(*args, **kwargs):
             for key in items:
                 if key not in kwargs:
-                    raise TypeError
+                    raise TypeError(ARG_IS_REQUIRED.format(key))
             return f(*args, **kwargs)
         return inner
     return wrapper
@@ -122,9 +200,10 @@ def returns(*return_types):
         @wraps(f)
         def inner(*args, **kwargs):
             ret = f(*args, **kwargs)
-            if not type(ret) in return_types:
-                raise TypeError
-            return ret
+            for i in return_types:
+                if isinstance(ret, i):
+                    return ret
+            raise TypeError(MUST_RETURN_TYPE.format(return_types))
         return inner
     return wrapper
 
@@ -154,10 +233,14 @@ def rejects(*positional, **named):
                 if isinstance(argtype, type):
                     if index < length:
                         if isinstance(args[index], argtype):
-                            raise TypeError
+                            raise TypeError(
+                                    MUST_NOT_BE_TYPE.format(item, argtype)
+                                    )
                     elif item in kws:
                         if isinstance(kws[item], argtype):
-                            raise TypeError
+                            raise TypeError(
+                                    MUST_NOT_BE_TYPE.format(item, argtype)
+                                    )
             return f(*args, **kwargs)
         return inner
     return wrapper
@@ -196,10 +279,14 @@ def accepts(*positional, **named):
                     # function.
                     if index < length:
                         if not isinstance(args[index], argtype):
-                            raise TypeError
+                            raise TypeError(
+                                    MUST_ACCEPT_TYPE.format(item, argtype)
+                                    )
                     elif item in kws:
                         if not isinstance(kws[item], argtype):
-                            raise TypeError
+                            raise TypeError(
+                                    MUST_ACCEPT_TYPE.format(item, argtype)
+                                    )
             return f(*args, **kwargs)
         return inner
     return wrapper
